/*
 * Copyright 2018 iserge.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.cesiumjs.cs.datasources.graphics.options;

import jsinterop.annotations.JsConstructor;
import jsinterop.annotations.JsPackage;
import jsinterop.annotations.JsProperty;
import jsinterop.annotations.JsType;
import org.cesiumjs.cs.core.Color;
import org.cesiumjs.cs.core.Math;
import org.cesiumjs.cs.datasources.properties.ConstantProperty;
import org.cesiumjs.cs.datasources.properties.Property;
import org.cesiumjs.cs.scene.enums.ShadowMode;

/**
 * @author Serge Silaev aka iSergio
 */
@JsType(isNative = true, namespace = JsPackage.GLOBAL, name = "Object")
public class CorridorGraphicsOptions {
    /**
     * A Property specifying the array of Cartesian3 positions that define the
     * centerline of the corridor.
     */
    @JsProperty
    public Property positions;
    /**
     * A numeric Property specifying the distance between the edges of the corridor.
     */
    @JsProperty
    public Property width;
    /**
     * A CornerType Property specifying the style of the corners. Default:
     * CornerType#ROUNDED
     */
    @JsProperty
    public Property cornerType;
    /**
     * A numeric Property specifying the altitude of the corridor relative to the
     * ellipsoid surface. Default: 0
     */
    @JsProperty
    public Property height;
    /**
     * A Property specifying what the height is relative to.
     */
    @JsProperty
    public Property heightReference;
    /**
     * A numeric Property specifying the altitude of the corridor's extruded face
     * relative to the ellipsoid surface.
     */
    @JsProperty
    public Property extrudedHeight;
    /**
     * A Property specifying what the extrudedHeight is relative to.
     */
    @JsProperty
    public Property extrudedHeightReference;
    /**
     * A boolean Property specifying the visibility of the corridor. Default: true
     */
    @JsProperty
    public Property show;
    /**
     * A boolean Property specifying whether the corridor is filled with the
     * provided material. Default: true
     */
    @JsProperty
    public Property fill;
    /**
     * A Property specifying the material used to fill the corridor. Default:
     * {@link org.cesiumjs.cs.core.Color#WHITE()}
     */
    @JsProperty
    public Property material;
    /**
     * A boolean Property specifying whether the corridor is outlined. Default:
     * false
     */
    @JsProperty
    public Property outline;
    /**
     * A Property specifying the Color of the outline. Default:
     * {@link Color#BLACK()}
     */
    @JsProperty
    public Property outlineColor;
    /**
     * A numeric Property specifying the width of the outline. Default: 1.0
     */
    @JsProperty
    public Property outlineWidth;
    /**
     * A numeric Property specifying the distance between each latitude and
     * longitude. Default: {@link Math#RADIANS_PER_DEGREE()}
     */
    @JsProperty
    public Property granularity;
    /**
     * An enum Property specifying whether the corridor casts or receives shadows
     * from each light source. Default: {@link ShadowMode#DISABLED()}
     */
    @JsProperty
    public Property shadows;
    /**
     * A Property specifying at what distance from the camera that this corridor
     * will be displayed.
     */
    @JsProperty
    public Property distanceDisplayCondition;
    /**
     * A Property specifying the zIndex of the corridor, used for ordering. Only has
     * an effect if height and extrudedHeight are undefined, and if the corridor is
     * static. Default: 0
     */
    @JsProperty
    public ConstantProperty<Integer> zIndex;

    /**
     * Options for {@link org.cesiumjs.cs.datasources.graphics.CorridorGraphics}
     */
    @JsConstructor
    public CorridorGraphicsOptions() {
    }
}
