/*
 * Copyright 2018 iserge.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.cesiumjs.cs.scene.providers.options;

import jsinterop.annotations.JsConstructor;
import jsinterop.annotations.JsPackage;
import jsinterop.annotations.JsProperty;
import jsinterop.annotations.JsType;
import org.cesiumjs.cs.core.Credit;
import org.cesiumjs.cs.core.Ellipsoid;
import org.cesiumjs.cs.core.Rectangle;

/**
 * @author Serge Silaev aka iSergio
 */
@JsType(isNative = true, namespace = JsPackage.GLOBAL, name = "Object")
public class OpenStreetMapImageryProviderOptions {
    /**
     * The OpenStreetMap server url. Default: https://a.tile.openstreetmap.org
     */
    @JsProperty
    public String url;
    /**
     * The file extension for images on the server. Default: png
     */
    @JsProperty
    public String fileExtension;
    /**
     * When true, request tiles at the 2x resolution for retina displays.
     * Default: false
     */
    @JsProperty
    public boolean retinaTiles;
    // TODO: Proxy
    // /**
    // * A proxy to use for requests. This object is expected to have a getURL
    // function which returns the proxied URL.
    // */
    // @JsProperty
    // public Proxy proxy;
    /**
     * The rectangle of the layer. Default: {@link Rectangle#MAX_VALUE()}
     */
    @JsProperty
    public Rectangle rectangle;
    /**
     * The minimum level-of-detail supported by the imagery provider.
     */
    @JsProperty
    public int minimumLevel;
    /**
     * The maximum level-of-detail supported by the imagery provider, or undefined
     * if there is no limit.
     */
    @JsProperty
    public int maximumLevel;
    /**
     * The ellipsoid. If not specified, the WGS84 ellipsoid is used.
     */
    @JsProperty
    public Ellipsoid ellipsoid;
    /**
     * A credit for the data source, which is displayed on the canvas. Default:
     * 'MapQuest, Open Street Map and contributors, CC-BY-SA'
     */
    @JsProperty
    public Credit credit;

    @JsConstructor
    public OpenStreetMapImageryProviderOptions() {
    }
}
